@@ -1,14 +1,24 @@
 #' @title Penalized quantile regression with fixed effects 
 #'
-#' @description Estimate parameters and tuning parameter.
-#'
+#' @description 
+#' Estimate regression parameters and tuning parameters for quantile, expectile, or M-quantile regression.  
+#' 
+#' \bold{Remarks:}  
+#' 
+#' 1. If the first column of `x` is entirely equal to 1, then the first element of `beta` represents the common intercept.  
+#'    Otherwise, there is no default common intercept (unlike the default behavior in `lm`).  
+#' 
+#' 2. If there is a common intercept and `effect` is `"fixed"` or `"lasso"`, a 'sum-to-zero constraint' is applied on the `alpha` parameters:  
+#'    \deqn{\sum_{i=1}^{n} \alpha_i = 0}  
+#'    This follows the approach in Danilevicz (2025).  
+#'  
 #' @param y      Numeric vector, outcome.
 #' @param x      Numeric matrix, covariates
 #' @param subj   Numeric vector, identifies the unit to which the observation belongs.
 #' @param tau    Numeric scalar between zero and one, identifies the percentile.
 #' @param effect Factor, "simple" simple regression, "fixed" regression with fixed effects, "lasso" penalized regression with fixed effects.
 #' @param c  Numeric, 0 is quantile, Inf is expectile, any number between zero and infinite is M-quantile.
-#  
+#'
 #' @return alpha       Numeric vector, intercepts' coefficients.
 #' @return beta        Numeric vector, exploratory variables' coefficients.
 #' @return lambda      Numeric, estimated lambda.
@@ -23,8 +33,7 @@
 #' @return Mat_alpha   Numeric matrix, intercepts' summary.
 #' @return Mat_beta    Numeric matrix, exploratory variables' summary.
 #' 
-#' @importFrom MASS stats graphics
-#' @import Rcpp RcppArmadillo
+#' @import MASS Rcpp RcppArmadillo
 #' 
 #' @examples 
 #' n = 10
@@ -41,10 +50,14 @@
 #' m1$Tab_beta
 #' 
 #' @references
+#' Danilevicz, I.M., Bondon, P., Reisen, V.A. (2025) "Adaptive LASSO Quantile Regression with Fixed Effects", Appl. Math. Model., xx (xx), <doi:10.1016/j.apm.2025.116600>
+#' 
+#' Danilevicz, I.M., Reisen, V.A., Bondon, P. (2024) "Expectile and M-quantile regression for panel data", Stat. Comput., 34 (97), <doi:10.1007/s11222-024-10396-7>
+#' 
 #' Koenker, R. (2004) "Quantile regression for longitudinal data", J. Multivar. Anal., 91(1): 74-89, <doi:10.1016/j.jmva.2004.05.006>
 #'      
 #' @export
-pqr =  function(x,y,subj,tau=0.5, effect="simple", c=1){
+pqr =  function(x,y,subj,tau=0.5, effect="fixed", c=0){
   penalty = choice_p(effect) 
   d  = ifelse(is.null(dim(x)[2]), 1, dim(x)[2])
   if(d==1) x = as.matrix(x)
@@ -124,14 +137,16 @@ pqr =  function(x,y,subj,tau=0.5, effect="simple", c=1){
 #' @param N Numeric integer, sample size.
 #' @param d Numeric integer, X number of columns.
 #' 
-#' @return parametric vector and residuals.
+#' @import stats
+#' @keywords internal
+#' @noRd
 optim_qr = function(beta,x,y,tau,N,d){
-  Opt    = stats::optim(par=beta, fn = loss_qr, method = "L-BFGS-B",  
+  Opt    = optim(par=beta, fn = loss_qr_simple, method = "L-BFGS-B",  
                  N=N, y=y, x=x, tau=tau, d=d)
   beta   = Opt$par
   if(d==1) res = y -  (x * beta)
   else     res = y -  (x %*% beta)   
-  return(list(alpha=0, beta=beta, lambda=0, res=res))
+  return(list(alpha=NA, beta=beta, lambda=0, res=res))
 }
 
 #' @title optim quantile regression with fixed effects
@@ -148,18 +163,22 @@ optim_qr = function(beta,x,y,tau,N,d){
 #' @param d Numeric integer, X number of columns.
 #' @param n Numeric integer, length of alpha.
 #' 
-#' @return parametric vector and residuals.
+#' @import stats
+#' @keywords internal
+#' @noRd
 optim_qrfe = function(beta,alpha,x,y,z,tau,N,d,n){
   M      = n
   theta  = c(beta,alpha)
-  Opt    = stats::optim(par=theta, fn = loss_qrfe, method = "L-BFGS-B",  
+  Opt    = optim(par=theta, fn = loss_qrfe, method = "L-BFGS-B",  
             n=N, y=y, x=x, z=z, tau=tau, d=d, mm=n)
   beta   = Opt$par[1:d]
   alpha  = Opt$par[(d+1):(d+n)]
+  # Sum-to-zero constraint if first column of x is all 1
+  if(all(x[,1] == 1)) alpha <- alpha - mean(alpha)
   if(d==1) res = y -  (z %*% alpha) -  (x * beta)
   else     res = y -  (z %*% alpha) -  (x %*% beta) 
-  sdu    = stats::sd(res, na.rm = TRUE)
-  sda    = stats::sd(alpha, na.rm = TRUE)
+  sdu    = sd(res, na.rm = TRUE)
+  sda    = sd(alpha, na.rm = TRUE)
   lambda = check_lambda(sdu/sda, 1/M, M)
   return(list(alpha=alpha, beta=beta, lambda=lambda, res=res))
 }
@@ -178,18 +197,22 @@ optim_qrfe = function(beta,alpha,x,y,z,tau,N,d,n){
 #' @param d Numeric integer, X number of columns.
 #' @param n Numeric integer, length of alpha.
 #' 
-#' @return parametric vector and residuals.
+#' @import stats
+#' @keywords internal
+#' @noRd
 optim_qrlasso = function(beta,alpha,x,y,z,tau,N,d,n){
   M       = n
   theta  = c(beta,alpha)
-  Opt    = stats::optim(par=theta, fn = loss_qrlasso, method = "L-BFGS-B",   
+  Opt    = optim(par=theta, fn = loss_qrlasso, method = "L-BFGS-B",   
                  n=N, y=y, x=x, z=z, tau=tau, d=d, mm=n,lambda=1)
   beta   = Opt$par[1:d]
   alpha  = Opt$par[(d+1):(d+n)]
+  # Sum-to-zero constraint if first column of x is all 1
+  if(all(x[,1] == 1)) alpha <- alpha - mean(alpha)
   if(d==1) res = y -  (z %*% alpha) -  (x * beta)
   else     res = y -  (z %*% alpha) -  (x %*% beta)   
-  sdu    = stats::sd(res, na.rm = TRUE)
-  sda    = stats::sd(alpha, na.rm = TRUE)
+  sdu    = sd(res, na.rm = TRUE)
+  sda    = sd(alpha, na.rm = TRUE)
   lambda = check_lambda(sdu/sda, 1/M, M)
   return(list(alpha=alpha, beta=beta, lambda=lambda, res=res))
 }
@@ -206,9 +229,11 @@ optim_qrlasso = function(beta,alpha,x,y,z,tau,N,d,n){
 #' @param d Numeric integer, X number of columns.
 #' @param c Numeric, positive real value.
 #' 
-#' @return parametric vector and residuals.
+#' @import stats
+#' @keywords internal
+#' @noRd
 optim_mqr = function(beta,x,y,tau,N,d,c){
-  Opt    = stats::optim(par=beta, fn = loss_mqr, method = "L-BFGS-B",  
+  Opt    = optim(par=beta, fn = loss_mqr, method = "L-BFGS-B",  
                  N=N, y=y, x=x, tau=tau, d=d, c=c)
   beta   = Opt$par[1:d]
   if(d==1) res = y -  (x * beta)
@@ -231,18 +256,22 @@ optim_mqr = function(beta,x,y,tau,N,d,c){
 #' @param n Numeric integer, length of alpha.
 #' @param c Numeric, positive real value.
 #' 
-#' @return parametric vector and residuals.
+#' @import stats
+#' @keywords internal
+#' @noRd
 optim_mqrfe = function(beta,alpha,x,y,z,tau,N,d,n,c){
   M      = n
   theta  = c(beta,alpha)
-  Opt    = stats::optim(par=theta, fn = loss_mqrfe, method = "L-BFGS-B", 
+  Opt    = optim(par=theta, fn = loss_mqrfe, method = "L-BFGS-B", 
                  n=N, y=y, x=x, z=z, tau=tau, d=d, mm=n, c=c)
   beta   = Opt$par[1:d]
   alpha  = Opt$par[(d+1):(d+n)]
+  # Sum-to-zero constraint if first column of x is all 1
+  if(all(x[,1] == 1)) alpha <- alpha - mean(alpha)
   if(d==1) res = y -  (z %*% alpha) -  (x * beta)
   else     res = y -  (z %*% alpha) -  (x %*% beta) 
-  sdu    = stats::sd(res, na.rm = TRUE)
-  sda    = stats::sd(alpha, na.rm = TRUE)
+  sdu    = sd(res, na.rm = TRUE)
+  sda    = sd(alpha, na.rm = TRUE)
   lambda = check_lambda(sdu/sda, 1/M, M)
   return(list(alpha=alpha, beta=beta, lambda=lambda, res=res))
 }
@@ -262,18 +291,22 @@ optim_mqrfe = function(beta,alpha,x,y,z,tau,N,d,n,c){
 #' @param n Numeric integer, length of alpha.
 #' @param c Numeric, positive real value.
 #' 
-#' @return parametric vector and residuals.
+#' @import stats
+#' @keywords internal
+#' @noRd
 optim_mqrlasso = function(beta,alpha,x,y,z,tau,N,d,n,c){
   M      = n
   theta  = c(beta,alpha)
-  Opt    = stats::optim(par=theta, fn = loss_mqrlasso, method = "L-BFGS-B",   
+  Opt    = optim(par=theta, fn = loss_mqrlasso, method = "L-BFGS-B",   
                  n=N, y=y, x=x, z=z, tau=tau, d=d, mm=n, c=c,lambda=1)
   beta   = Opt$par[1:d]
   alpha  = Opt$par[(d+1):(d+n)]
+  # Sum-to-zero constraint if first column of x is all 1
+  if(all(x[,1] == 1)) alpha <- alpha - mean(alpha)
   if(d==1) res = y -  (z %*% alpha) -  (x * beta)
   else     res = y -  (z %*% alpha) -  (x %*% beta)   
-  sdu    = stats::sd(res, na.rm = TRUE)
-  sda    = stats::sd(alpha, na.rm = TRUE)
+  sdu    = sd(res, na.rm = TRUE)
+  sda    = sd(alpha, na.rm = TRUE)
   lambda = check_lambda(sdu/sda, 1/M, M)
   return(list(alpha=alpha, beta=beta, lambda=lambda, res=res))
 }
@@ -289,9 +322,11 @@ optim_mqrlasso = function(beta,alpha,x,y,z,tau,N,d,n,c){
 #' @param N Numeric integer, sample size.
 #' @param d Numeric integer, X number of columns.
 #' 
-#' @return parametric vector and residuals.
+#' @import stats
+#' @keywords internal
+#' @noRd
 optim_er = function(beta,x,y,tau,N,d){
-  Opt    = stats::optim(par=beta, fn = loss_er, method = "L-BFGS-B",  
+  Opt    = optim(par=beta, fn = loss_er, method = "L-BFGS-B",  
                  N=N, y=y, x=x, tau=tau, d=d)
   beta   = Opt$par
   if(d==1) res = y -  (x * beta)
@@ -313,18 +348,22 @@ optim_er = function(beta,x,y,tau,N,d){
 #' @param d Numeric integer, X number of columns.
 #' @param n Numeric integer, length of alpha.
 #' 
-#' @return parametric vector and residuals.
+#' @import stats
+#' @keywords internal
+#' @noRd
 optim_erfe = function(beta,alpha,x,y,z,tau,N,d,n){
   M      = n
   theta  = c(beta,alpha)
-  Opt    = stats::optim(par=theta, fn = loss_erfe, method = "L-BFGS-B",  
+  Opt    = optim(par=theta, fn = loss_erfe, method = "L-BFGS-B",  
                  n=N, y=y, x=x, z=z, tau=tau, d=d, mm=n)
   beta   = Opt$par[1:d]
   alpha  = Opt$par[(d+1):(d+n)]
+  # Sum-to-zero constraint if first column of x is all 1
+  if(all(x[,1] == 1)) alpha <- alpha - mean(alpha)
   if(d==1) res = y -  (z %*% alpha) -  (x * beta)
   else     res = y -  (z %*% alpha) -  (x %*% beta) 
-  sdu    = stats::sd(res, na.rm = TRUE)
-  sda    = stats::sd(alpha, na.rm = TRUE)
+  sdu    = sd(res, na.rm = TRUE)
+  sda    = sd(alpha, na.rm = TRUE)
   lambda = check_lambda(sdu/sda, 1/M, M)
   return(list(alpha=alpha, beta=beta, lambda=lambda, res=res))
 }
@@ -343,18 +382,22 @@ optim_erfe = function(beta,alpha,x,y,z,tau,N,d,n){
 #' @param d Numeric integer, X number of columns.
 #' @param n Numeric integer, length of alpha.
 #' 
-#' @return parametric vector and residuals.
+#' @import stats
+#' @keywords internal
+#' @noRd
 optim_erlasso = function(beta,alpha,x,y,z,tau,N,d,n){
   M      = n
   theta  = c(beta,alpha)
-  Opt    = stats::optim(par=theta, fn = loss_erlasso, method = "L-BFGS-B",   
+  Opt    = optim(par=theta, fn = loss_erlasso, method = "L-BFGS-B",   
                  n=N, y=y, x=x, z=z, tau=tau, d=d, mm=n,lambda=1)
   beta   = Opt$par[1:d]
   alpha  = Opt$par[(d+1):(d+n)]
+  # Sum-to-zero constraint if first column of x is all 1
+  if(all(x[,1] == 1)) alpha <- alpha - mean(alpha)
   if(d==1) res = y -  (z %*% alpha) -  (x * beta)
   else     res = y -  (z %*% alpha) -  (x %*% beta)   
-  sdu    = stats::sd(res, na.rm = TRUE)
-  sda    = stats::sd(alpha, na.rm = TRUE)
+  sdu    = sd(res, na.rm = TRUE)
+  sda    = sd(alpha, na.rm = TRUE)
   lambda = check_lambda(sdu/sda, 1/M, M)
   return(list(alpha=alpha, beta=beta, lambda=lambda, res=res))
 }
@@ -363,26 +406,17 @@ optim_erlasso = function(beta,alpha,x,y,z,tau,N,d,n){
 #'
 #' @param x Numeric vector.
 #' 
-#' @return y vector, kernel density estimation.
-#' 
-#' @examples 
-#' x = rnorm(10)
-#' f_den(x)
-#' 
+#' @import stats
+#' @keywords internal
+#' @noRd
 f_den = function(x){
-  inf = 1e-08
-  n  = length(x)
-  dh = stats::density(x, kernel = "epanechnikov")
-  dx = dh$x
-  dy = dh$y
-  M  = length(dx)
-  y  = rep(max(min(dy),inf),n)
-  for(i in 1:n){
-    for(j in 2:(M-2)){
-      if(x[i]>=dx[j-1] && x[i]<=dx[j+1]) y[i] = dy[j] 
-    }
-  }
-  return(y)  
+  inf <- 1e-08
+  # Compute density once
+  dh <- density(x, kernel = "epanechnikov")
+  # Interpolate density values for x
+  y <- approx(dh$x, dh$y, xout = x, rule = 2)$y
+  # Ensure minimum > inf
+  return(pmax(y, inf))  
 }
 
 #' @title Covariance
@@ -399,7 +433,9 @@ f_den = function(x){
 #' @param penalty Numeric, 1 quantile regression, 2 quantile regression with fixed effects, 3 Lasso quantile regression with fixed effects
 #' @param c Numeric, tuning
 #' 
-#' @return a list with two matrices: sig2_alpha (which is the matrix of covariance of estimated alpha) and sig2_beta (which is the matrix of covariance of estimated beta)
+#' @import MASS
+#' @keywords internal
+#' @noRd
 q_cov = function(n, N, d, Z, X, tau, res, penalty, c){
   omega = tau*(1-tau)
   inf = 1/(10^8)
@@ -494,7 +530,9 @@ q_cov = function(n, N, d, Z, X, tau, res, penalty, c){
 #' @param sig2 Numeric vector.
 #' @param kind Numeric, 1 means alpha, 2 means beta
 #'
-#' @return a list with a dataframe Core and a matrix Matx, both display the same information
+#' @import stats
+#' @keywords internal
+#' @noRd
 f_tab = function(N, n, d, theta, sig2, kind){
   inf = 1e-08
   m = N/n
@@ -509,7 +547,7 @@ f_tab = function(N, n, d, theta, sig2, kind){
   infb  = theta - (len * SE) 
   supb  = theta + (len * SE)
   zval  = theta/SE
-  pval  = 2 * stats::pnorm(abs(theta/SE), lower.tail = F)
+  pval  = 2 * pnorm(abs(theta/SE), lower.tail = F)
   sig0  = sgf(as.vector(pval))
   Matx  = matrix(cbind(theta, SE, infb, supb, zval, pval), ncol=6, nrow=p)
   Core  = data.frame(cbind(theta, SE, infb, supb, zval, pval, sig0))
@@ -529,14 +567,8 @@ f_tab = function(N, n, d, theta, sig2, kind){
 #'
 #' @param x Numeric vector.
 #' 
-#' @return y vector Factor, symbol flag of significant p-values.
-#' 
-#' @examples 
-#' n = 10
-#' pvalue = rgamma(10,1,10)
-#' sgf(pvalue) 
-#' 
-#' @return a vector of Factors, i.e., the symbols to help p-value interpretation
+#' @keywords internal
+#' @noRd
 sgf = function(x){
   m = length(x)
   y = rep(" ", m)
@@ -556,10 +588,12 @@ sgf = function(x){
 #' @param infb Numeric, lower bound of lambda.
 #' @param supb Numeric, upper bound of lambda.
 #' 
-#' @return lambda Numeric, valid value of lambda.
+#' @import stats
+#' @keywords internal
+#' @noRd
 check_lambda = function(lambda, infb, supb){
   if(is.na(lambda) || is.null(lambda)){
-    lambda = stats::runif(1,infb, supb)
+    lambda = runif(1,infb, supb)
   }else{
     if(lambda <= infb){
       lambda = infb;
@@ -575,7 +609,8 @@ check_lambda = function(lambda, infb, supb){
 #'
 #' @param effect Factor, simple, fixed or lasso.
 #' 
-#' @return penalty Numeric, 1, 2 and 3.
+#' @keywords internal
+#' @noRd
 choice_p = function(effect){
   penalty = 1
   if(effect == "fixed") penalty = 2
@@ -608,33 +643,19 @@ choice_p = function(effect){
 #' clean_data(y=y, x=x, id=subj)  
 #'  
 clean_data = function(y, x, id){
-  n = max(id)
-  N = length(id)
-  d = dim(x)[2]
-  ynew = NULL
-  xnew = NULL
-  nj   = NULL
-  inew = NULL
-  for(j in 1:n){
-    n0 = length(ynew)
-    for(i in 1:N){
-      if(id[i] == j){
-        if( any(c(is.null(y[i]), is.null(x[i,]), is.na(y[i]), is.na(x[i,])))){
-        } else {
-            ynew = c(ynew, y[i])
-            xnew = matrix(rbind(xnew, x[i,]), ncol=d)
-            inew = c(inew, id[i])
-        }  
-      }
-    }
-    n1 = length(ynew) - n0
-    if(n1>0) nj = rbind(nj, n1)  
-  }
-  y = as.vector(ynew)
-  x = as.matrix(xnew)
-  nj = as.vector(nj)
-  id = as.vector(inew)
-  return(list(y=y,x=x,nj=nj,id=id))
+  # 1. Define valid rows: non-NULL, non-NA
+  # Since y[i] or x[i,] cannot be NULL inside vectors/matrices,
+  # we just check for NA.
+  valid <- !is.na(y) & apply(x, 1, function(row) !any(is.na(row)))
+  # Keep only valid rows
+  ynew <- y[valid]
+  xnew <- x[valid, , drop = FALSE]
+  idnew <- id[valid]
+  # 2. Count observations per id in the filtered data
+  nj <- tabulate(idnew, nbins = max(id))
+  # Remove zeros (for ids that disappeared)
+  nj <- nj[nj > 0]
+  return(list(y = ynew, x = xnew, nj = nj, id = idnew))
 }
 
 #' @title Print an PQR
@@ -644,7 +665,8 @@ clean_data = function(y, x, id){
 #' @param x An object of class "PQR"
 #' @param ... further arguments passed to or from other methods.
 #' 
-#' @return None
+#' @keywords internal
+#' @noRd
 print.PQR = function(x,...){
   if(x$penalty == 1){
     if(x$c==0){
@@ -780,6 +802,7 @@ mpqr = function(x,y,subj,tau=1:9/10, effect="simple", c=0){
 #' Beta = mpqr(x,y,subj,tau=1:9/10, effect="lasso", c = Inf)
 #' plot_taus(Beta,tau=1:9/10,D=1)
 #'
+#' @import graphics
 #' @export
 plot_taus = function(Beta, tau=1:9/10, D, col=2, lwd=1, lty=2, pch=16, cex.axis=1, cex.lab=1, main="", shadow="gray90"){
   ntau  = dim(Beta)[1]
